#include "massren.h"
#include <iostream>
#include <errno.h>
#include "cfile.h"
#include "config.h"
#include <stdlib.h>

MassRenamer::
MassRenamer(const std::string &name) :
    ok_pix_(DATA_PREFIX "data/ok.png"), ko_pix_(DATA_PREFIX "data/ko.png"), 
    yes_pix_(DATA_PREFIX "data/bullet_green.png"), no_pix_(DATA_PREFIX "data/bullet_red.png")
{
    Load(DATA_PREFIX "data/gui.xml");

    source_ = Get<gtk::Entry>("entry_keywords");
    dest_ = Get<gtk::Entry>("entry_output");
    main_ = Get<gtk::Window>("window_main");
    dir_ = Get<gtk::FileChooserButton>("filechooserbutton_source");
    list_ = Get<gtk::ListStore>("liststore_results");
    destdir_ = Get<gtk::FileChooserButton>("filechooserbutton_dest");
    usedest_ = Get<gtk::ToggleButton>("checkbutton_samedir");

    if (!source_ || !dest_ || !main_ || !dir_ || !list_ ||
        !destdir_ || !usedest_) {
        std::cerr << "XML Error, quitting!\n"; 
        ::exit(0);
    }

    if (!name.empty()) {
        dir_->CurrentFolder(name);
        destdir_->CurrentFolder(name);
    }

    main_->OnDelete(&Application::QuitLoop, dynamic_cast<Application *>(this), true);

    Get<gtk::Button>("button_quit")->OnClick(&Application::QuitLoop, dynamic_cast<Application *>(this));

    Get<gtk::Button>("button_ok")->OnClick(&MassRenamer::rename, this);
    source_->OnChanged(&MassRenamer::changed, this);
    dest_->OnChanged(&MassRenamer::changed, this);
    usedest_->OnToggle(&MassRenamer::usedest_changed, this);
    Get<gtk::CellRendererToggle>("cellrenderertoggle1")->OnToggled(&MassRenamer::toggled, this);
    // a timer to check if we changed the source or destination directory
    AddTimer(250, &MassRenamer::timer, this);

    main_->Title("Mass Renamer " PROGRAM_VERSION);
    main_->Show();
}

void MassRenamer::
rename()
{
    std::string src_dir  = dir_->Filename(), dest_dir;
  
    if (*src_dir.rbegin() != '/')
        src_dir.push_back('/');

    if (usedest_->Active()) {
        dest_dir  = destdir_->Filename();
        if (*dest_dir.rbegin() != '/')
            dest_dir.push_back('/');
    }
    else
        dest_dir = src_dir;

    std::map<std::string, std::string> failed;

    for(gtk::TreeIter it = list_->First(); list_->IsValid(it); list_->Next(it)) {
        int renamethis = 0;
        list_->GetValue(it, 4, renamethis);

        if (!renamethis)
            continue;

        std::string old_name, new_name;
        list_->GetValue(it, 1, old_name);
        list_->GetValue(it, 2, new_name);

        std::string o = src_dir + old_name, n = dest_dir + new_name;
        // no need to rename files that have already the right name.
        if (n == o) {
            list_->Set(it, 0, ok_pix_.Obj(), -1);
            continue;
        }
        if (::rename(o.c_str(), n.c_str()) != 0) {
            failed[old_name] = ::strerror(errno);
            list_->Set(it, 0, ko_pix_.Obj(), -1);
        }
        else
            list_->Set(it, 0, ok_pix_.Obj(), -1);
    }

    if (!failed.empty())
        gtk::MessageDialog(main_, gtk::DialogModal, gtk::MessageInfo, gtk::ButtonsOk,
                           "Operation completed with %d errors", failed.size()).Run();

    else
        gtk::MessageDialog(main_, gtk::DialogModal, gtk::MessageInfo, gtk::ButtonsOk,
                           "Operation completed").Run();

    actual_ = "fakedirname";
    dest_->Set("");
    changed();
}

void MassRenamer::
usedest_changed()
{
    if (gtk::Widget *w = Get<gtk::Widget>("hbox_dest"))
        w->Show(!usedest_->Active());
}

void MassRenamer::
toggled(const std::string &path)
{
    gtk::TreeIter it;
    if (list_->Get(it, path)) {
        int value;
        list_->GetValue(it, 4, value);
        value ^=1;
        list_->SetValue(it, 4, value);
    }
}

void MassRenamer::
timer()
{
    if (actual_ != dir_->Filename()) {
        actual_ = dir_->Filename();
        rencore_.build_file_cache(actual_);
    // if the source directory changes we have to recalculate everything...
        changed();
    }
}

void MassRenamer::
populate_list(const RenCore::FileMap &files)
{
    int duplicated = 0, total = 0;
    std::set<std::string> dest_names;
    list_->Clear();
    for (RenCore::FileCIt it = files.begin(); it != files.end(); ++it)  {
        gtk::Pixbuf *pixbuf;
        std::string dest_name = it->second.first;

        // skip empty entries (failed substitution)
        if (dest_name.empty())
            continue;

        total++;

        if (dest_names.find(dest_name) != dest_names.end()) {
            duplicated++;
            pixbuf = &no_pix_;
        }
        else {
            pixbuf = &yes_pix_;
            dest_names.insert(dest_name);
        }

        list_->AddTail(0, pixbuf->Obj(),
                       1, it->first.c_str(), 
                       2, dest_name.c_str(), 
                       3, it->second.second,     
                       4, pixbuf == &yes_pix_, 5, TRUE, // check only if it's not duplicated
                       -1);
    }

    if (gtk::Label *l = Get<gtk::Label>("label_results")) {
        if (duplicated > 0)
            l->SetF("<span foreground=\"red\"><b>The operation will rename %d items with %d duplicate result names.</b></span>", total, duplicated);
        else if (total < files.size())
            l->SetF("<span foreground=\"#0000a0\"><b>The operation will rename %d items (%d do not match numbering rules).</b></span>", total, files.size() - total);
        else            
            l->SetF("<span foreground=\"#00a000\"><b>The operation will rename %d items.</b></span>", total);
    }
}


void MassRenamer::
changed()
{
    // do nothing if we don't have source and dest with some valid values
    if (dest_->Get().empty() || source_->Get().empty()) {
        if (gtk::Label *l = Get<gtk::Label>("label_results"))
            l->Set("");
        list_->Clear();
        return;
    }

    RenCore::FileMap files;

    rencore_.set_keys(source_->Get());
    rencore_.dest(dest_->Get());
    rencore_.get_files(files);

    populate_list(files);
}

int main(int argc, char *argv[])
{
    char buffer[256];
    ::getcwd(buffer, sizeof(buffer));
    if (argc > 1) {
        CDir d(argv[1]);

        if (d.good()) 
            ::strncpy(buffer, argv[1], sizeof(buffer));
    }

    MassRenamer mass(buffer);

    mass.Run();
}
