#include "rencore.h"
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include "config.h"
#include <errno.h>
#include <string.h>

void usage(const char *header)
{
    std::cout << header << '\n'
              << "Usage: mr [opts] <keys> <pattern>\n"
                 " keys    - comma separated list of keywords (case insensitive), at least one key"
                 "           must be specified. Example: big,bang\n"
                 " pattern - destination radix, may contain %s and %e for two optional\n"
                 "           numeric values. Example: Big.Bang.Theory.%sx%e\n"
                 "options:\n"
                 " -d dir  - source directory (if not specified defaults to the current directory)\n"
                 " -o dir  - destination directory (must exists and be in the same filesystem of\n"
                 "           the source), if not specified defaults to the source directory.\n"
                 " -v      - verbose mode\n"
                 " -h      - shows this page\n"
                 " -n      - not interactive, do not ask for confirmation before proceeding.\n\n"
                 "Example:\n"
                 " in 'test' dir: The_walking_DEAD_2_10.avi, WalkingDead-s02-10.srt\n"
                 " mr -d test walking,dead Walking.Dead.%sx%e\n"
                 " in 'test' dir: Walking.Dead.2x10.avi Walking.Dead.2x10.srt\n"
              << std::endl;
}

int main(int argc, char *argv[])
{   
    int opt;
    bool interactive = true, verbose = false;
    std::string keys, pattern;
    std::string src_dir, dest_dir;
    std::cout << "Mass Renamer " PROGRAM_VERSION "\n" << std::endl;

    while ((opt =getopt(argc, argv, "Hvhno:d:")) != -1) {
        switch (opt) {
            case 'o':
                dest_dir = optarg;
                break;
            case 'd':
                src_dir = optarg;
                break;
            case 'v':
                verbose = true;
                break;
            case 'h':
                usage("Program arguments:");
                return 0;
            case 'n':
                interactive = false;
                break;
        }
    }

    if (optind >= (argc - 1)) {
        usage("Invalid parameters:");
        return 0;
    }
    keys = argv[optind];
    pattern = argv[optind+1];

    if (src_dir.empty())
        src_dir = ".";
    if (dest_dir.empty())
        dest_dir = src_dir;

    if (verbose) {
        std::cerr << "Working with the following parameters"
                     "\nInput Directory : " << src_dir <<
                     "\nOutput Directory: " << dest_dir <<
                     "\nKeywords        : " << keys <<
                     "\nName Template   : " << pattern << "\n" << std::endl;
    }
    
    RenCore ren;

    ren.build_file_cache(src_dir);
    ren.dest(pattern);

    if (verbose)
        std::cerr << "Built directory index for " << src_dir << " of " << ren.cache_size() << " files.\n";

    if (!ren.set_keys(keys)) {
        std::cerr << "Invalid keywords: " << keys << '\n';
        return -1;
    }

    if (verbose)
        std::cerr << "Parsed a total of " << ren.key_size() << " keys.\n";

    RenCore::FileMap files;
    ren.get_files(files);

    if (files.empty()) {
        std::cerr << "No file matching your keywords, quitting.\n";
        return -1;
    }

    if (verbose)
        std::cerr << files.size() << " files are matching your keywords:\n";

    for (RenCore::FileCIt it = files.begin(); it != files.end(); ++it) {
        std::cout << std::setw(30) << it->first << " -> " 
                  << std::setw(30) << it->second.first << " (" << it->second.second << ")\n";
    }

    if (interactive) {
        std::cout << "\nExecute this operation (y/N)?" << std::endl;
        std::string result;
        std::cin >> result;

        if (result[0] != 'y' || result.size() != 1) {
            std::cerr << "\nOperation aborted.\n";
            return 0;
        }
    }
    if (!ren.perform(files, src_dir, dest_dir)) {
            std::cerr << "\nOperation completed with errors: " << ::strerror(errno) << "\n" << std::endl;
            return 0;
    }
    else
        std::cout << "Operation completed." << std::endl; 
}
