#include "rencore.h"
#include "cfile.h"
#include <sys/stat.h>
#include <algorithm>
#include <stdlib.h>
#include <sstream>
#include <iomanip>

bool RenCore::
set_keys(const std::string &src)
{
    keys_.clear();
    std::string key;
    for (std::string::const_iterator it = src.begin(); it != src.end(); ++it) {
        if (*it != ' ' || *it == ',')
            key.push_back(std::tolower(*it));
        else
            if (!key.empty()) {
                keys_.insert(key);
                key.erase();
            }
    }
    if (!key.empty())
        keys_.insert(key);

//    std::cerr << keys_.size() << " keys found.\n";
    return !keys_.empty();
}


bool RenCore::
perform(const FileMap &files, std::string src_dir, std::string dest_dir)
{
    bool ok = true;
    if (*src_dir.rbegin() != '/')
        src_dir.push_back('/');
    if (*dest_dir.rbegin() != '/')
        dest_dir.push_back('/');

    for (FileCIt it = files.begin(); it != files.end(); ++it) {
        std::string o = src_dir + it->first, n = dest_dir + it->second.first;

        // no need to rename, skip!
        if (o == n)
            continue;

        if (::rename(o.c_str(), n.c_str()) != 0) 
            ok = false;
    }
    return ok;
}

void RenCore::
build_file_cache(const std::string &dirname)
{
    files_.clear();
    CDir d(dirname);

    if (d.good()) {
        std::string name;

        while (d.get(name)) {
            struct stat stbuf;

            std::string fullpath = dirname;
            if (*fullpath.rbegin() != '/')
                fullpath.push_back('/');

            fullpath.append(name);

            // accept only regular files
            if (::stat(fullpath.c_str(), &stbuf) == 0 && S_ISREG(stbuf.st_mode)) 
                files_[name] = stbuf.st_size;
        }
    }
//    std::cerr << "Directory changed to <" << dirname << ">, " << files_.size() << " files cached!\n";
}

static bool find_number(const std::string &s, int *results, int total)
{
    bool found = false;
    int results_found = 0;
    std::string result;

    for (int i = 0; i < s.length(); ++i) {
        if (isdigit(s[i])) {
            result.push_back(s[i]);
            found = true;
        }
        else if (found) {
            results[results_found++] = ::atoi(result.c_str());
            result.clear();
            found = false;

            if (results_found == total)
                return true;
        }
    }
    if (!result.empty()) 
        results[results_found++] = ::atoi(result.c_str());

    return results_found == total;
}

template <typename T>
std::string to_string(T val, int len = -1, char filler = '0') {
    std::ostringstream os;
    if (len == -1)
        os << val;
    else 
        os << std::setfill(filler) << std::setw(len) << val;

    return os.str();
}

std::string RenCore::
make_new_name(const std::string &orig) {
    std::string result = dest_;
    std::string ext;
    int numbers = 0;
    std::string::size_type p = orig.find_last_of('.');

    if (p != std::string::npos)
        ext = orig.substr(p);
    std::string::size_type spos = result.find("%s");
    std::string::size_type epos = result.find("%e");

    if (spos != std::string::npos)
        numbers++;
    if (epos != std::string::npos) {
        // sprintf format wants only %s!
        result[epos + 1] = 's';
        numbers++;
    }

    char buffer[1024];

    switch (numbers) {
        case 1:
            {
                int n;
                if (find_number(orig, &n, 1)) {
                    snprintf(buffer, sizeof(buffer), result.c_str(), to_string(n, 2).c_str());
                    result = buffer;
                }
                else
                    return "";
            }
            break;
        case 2:
            {
                int n[2];
                if (find_number(orig, n, 2)) {
                    fprintf(stderr, "%d - %d\n", n[0], n[1]);
                    snprintf(buffer, sizeof(buffer), result.c_str(), to_string(n[0], 1).c_str(), to_string(n[1], 2).c_str());
                    result = buffer;
                }
                else
                    return "";
            }
            break;
    }

    result.append(ext);
//    std::cerr << "Built new name for " << orig << " -> " << result << '\n';
    return result;
}

template<typename charT>
struct my_equal {
    bool operator()(charT ch1, charT ch2) {
        return std::tolower(ch1) == ch2;
    }
};

bool RenCore::
match_keys(const std::string &line)
{
     for (std::set<std::string>::const_iterator it = keys_.begin(); it != keys_.end(); ++it)
        if (std::search( line.begin(), line.end(), it->begin(), it->end(), my_equal<std::string::value_type>() ) == line.end())
             return false;

     return true;
}


void RenCore::
get_files(FileMap &files)
{
    for (CacheCIt it = files_.begin(); it != files_.end(); ++it) 
        if (match_keys(it->first)) 
            files[it->first] = std::make_pair(make_new_name(it->first), it->second);
}

